import _objectWithoutPropertiesLoose from '@babel/runtime/helpers/objectWithoutPropertiesLoose'
import mitt from 'mitt'
import { graphqlFetchOptions } from './graphqlFetchOptions'
import { hashObject } from './hashObject'
export var GraphQL = function GraphQL(_temp) {
  var _this = this

  var _ref = _temp === void 0 ? {} : _temp,
    _ref$cache = _ref.cache,
    _cache = _ref$cache === void 0 ? {} : _ref$cache,
    _ref$logErrors = _ref.logErrors,
    logErrors = _ref$logErrors === void 0 ? true : _ref$logErrors

  this.reset = function(exceptFetchOptionsHash) {
    var fetchOptionsHashes = Object.keys(_this.cache)
    if (exceptFetchOptionsHash)
      fetchOptionsHashes = fetchOptionsHashes.filter(function(hash) {
        return hash !== exceptFetchOptionsHash
      })
    fetchOptionsHashes.forEach(function(fetchOptionsHash) {
      return delete _this.cache[fetchOptionsHash]
    })

    _this.emit('reset', {
      exceptFetchOptionsHash: exceptFetchOptionsHash
    })
  }

  this.request = function(_ref2, fetchOptionsHash) {
    var url = _ref2.url,
      options = _objectWithoutPropertiesLoose(_ref2, ['url'])

    var requestCache = {}
    var fetcher =
      typeof fetch === 'function'
        ? fetch
        : function() {
            return Promise.reject(
              new Error('Global fetch API or polyfill unavailable.')
            )
          }
    var cache = (_this.requests[fetchOptionsHash] = fetcher(url, options))
      .then(
        function(response) {
          if (!response.ok)
            requestCache.httpError = {
              status: response.status,
              statusText: response.statusText
            }
          return response.json().then(
            function(_ref3) {
              var errors = _ref3.errors,
                data = _ref3.data
              if (!errors && !data)
                requestCache.parseError = 'Malformed payload.'
              if (errors) requestCache.graphQLErrors = errors
              if (data) requestCache.data = data
            },
            function(_ref4) {
              var message = _ref4.message
              requestCache.parseError = message
            }
          )
        },
        function(_ref5) {
          var message = _ref5.message
          requestCache.fetchError = message
        }
      )
      .then(function() {
        _this.cache[fetchOptionsHash] = requestCache
        delete _this.requests[fetchOptionsHash]

        _this.emit('cache', {
          fetchOptionsHash: fetchOptionsHash
        })

        var fetchError = requestCache.fetchError,
          httpError = requestCache.httpError,
          parseError = requestCache.parseError,
          graphQLErrors = requestCache.graphQLErrors

        if (
          _this.logErrors &&
          (fetchError || httpError || parseError || graphQLErrors)
        ) {
          console.groupCollapsed(
            'GraphQL request (hash \u201C' +
              fetchOptionsHash +
              '\u201D) errors:'
          )

          if (fetchError) {
            console.groupCollapsed('Fetch:')
            console.log(fetchError)
            console.groupEnd()
          }

          if (httpError) {
            console.groupCollapsed('HTTP:')
            console.log('Status: ' + httpError.status)
            console.log('Text: ' + httpError.statusText)
            console.groupEnd()
          }

          if (parseError) {
            console.groupCollapsed('Parse:')
            console.log(parseError)
            console.groupEnd()
          }

          if (graphQLErrors) {
            console.groupCollapsed('GraphQL:')
            graphQLErrors.forEach(function(_ref6) {
              var message = _ref6.message
              return console.log(message)
            })
            console.groupEnd()
          }

          console.groupEnd()
        }

        return requestCache
      })

    _this.emit('fetch', {
      fetchOptionsHash: fetchOptionsHash,
      cache: cache
    })

    return cache
  }

  this.query = function(_ref7) {
    var operation = _ref7.operation,
      fetchOptionsOverride = _ref7.fetchOptionsOverride,
      resetOnLoad = _ref7.resetOnLoad
    var fetchOptions = graphqlFetchOptions(operation)
    if (fetchOptionsOverride) fetchOptionsOverride(fetchOptions)
    var fetchOptionsHash = hashObject(fetchOptions)

    var request =
      _this.requests[fetchOptionsHash] ||
      _this.request(fetchOptions, fetchOptionsHash)

    if (resetOnLoad)
      request.then(function() {
        return _this.reset(fetchOptionsHash)
      })
    return {
      fetchOptionsHash: fetchOptionsHash,
      cache: _this.cache[fetchOptionsHash],
      request: request
    }
  }

  var _mitt = mitt(),
    on = _mitt.on,
    off = _mitt.off,
    emit = _mitt.emit

  this.on = on
  this.off = off
  this.emit = emit
  this.cache = _cache
  this.requests = {}
  this.logErrors = logErrors
}
