'use strict'

var _interopRequireDefault = require('@babel/runtime/helpers/interopRequireDefault')

exports.__esModule = true
exports.GraphQLQuery = void 0

var _objectAssign = _interopRequireDefault(require('object-assign'))

var _inheritsLoose2 = _interopRequireDefault(
  require('@babel/runtime/helpers/inheritsLoose')
)

var _fastDeepEqual = _interopRequireDefault(require('fast-deep-equal'))

var _propTypes = _interopRequireDefault(require('prop-types'))

var _react = _interopRequireDefault(require('react'))

var _GraphQL = require('./GraphQL')

var _graphqlFetchOptions = require('./graphqlFetchOptions')

var _hashObject = require('./hashObject')

var GraphQLQuery = (function(_React$Component) {
  ;(0, _inheritsLoose2.default)(GraphQLQuery, _React$Component)

  function GraphQLQuery(props) {
    var _this

    _this = _React$Component.call(this, props) || this

    _this.onFetch = function(_ref) {
      var fetchOptionsHash = _ref.fetchOptionsHash
      if (fetchOptionsHash === _this.state.fetchOptionsHash)
        _this.setState({
          loading: true
        })
    }

    _this.onCache = function(_ref2) {
      var fetchOptionsHash = _ref2.fetchOptionsHash
      if (fetchOptionsHash === _this.state.fetchOptionsHash)
        _this.setState({
          loading: false,
          requestCache: _this.props.graphql.cache[fetchOptionsHash]
        })
    }

    _this.onReset = function(_ref3) {
      var exceptFetchOptionsHash = _ref3.exceptFetchOptionsHash
      if (exceptFetchOptionsHash !== _this.state.fetchOptionsHash)
        if (_this.props.loadOnReset) _this.load()
        else
          _this.setState({
            requestCache: null
          })
    }

    _this.query = function() {
      return _this.props.graphql.query({
        operation: _this.props.operation,
        fetchOptionsOverride: _this.props.fetchOptionsOverride,
        resetOnLoad: _this.props.resetOnLoad
      })
    }

    _this.load = function() {
      var _this$query = _this.query(),
        fetchOptionsHash = _this$query.fetchOptionsHash,
        cache = _this$query.cache,
        request = _this$query.request

      _this.setState({
        loading: true,
        fetchOptionsHash: fetchOptionsHash,
        cache: cache
      })

      return request
    }

    _this.state = {
      loading: props.loadOnMount
    }

    if (props.loadOnMount) {
      var fetchOptions = (0, _graphqlFetchOptions.graphqlFetchOptions)(
        props.operation
      )
      if (props.fetchOptionsOverride) props.fetchOptionsOverride(fetchOptions)
      _this.state.fetchOptionsHash = (0, _hashObject.hashObject)(fetchOptions)
      _this.state.requestCache =
        props.graphql.cache[_this.state.fetchOptionsHash]
    }

    props.graphql.on('fetch', _this.onFetch)
    props.graphql.on('cache', _this.onCache)
    props.graphql.on('reset', _this.onReset)
    return _this
  }

  var _proto = GraphQLQuery.prototype

  _proto.componentDidMount = function componentDidMount() {
    if (this.props.loadOnMount) this.load()
  }

  _proto.componentDidUpdate = function componentDidUpdate(_ref4) {
    var operation = _ref4.operation
    if (!(0, _fastDeepEqual.default)(operation, this.props.operation))
      if (this.props.loadOnMount) this.load()
      else
        this.setState({
          fetchOptionsHash: null,
          requestCache: null
        })
  }

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.props.graphql.off('fetch', this.onFetch)
    this.props.graphql.off('cache', this.onCache)
    this.props.graphql.off('reset', this.onReset)
  }

  _proto.render = function render() {
    if (
      this.props.graphql.ssr &&
      this.props.loadOnMount &&
      !this.state.requestCache
    ) {
      this.query()
      return null
    }

    return this.props.children(
      (0, _objectAssign.default)(
        {
          load: this.load,
          loading: this.state.loading
        },
        this.state.requestCache
      )
    )
  }

  return GraphQLQuery
})(_react.default.Component)

exports.GraphQLQuery = GraphQLQuery
GraphQLQuery.propTypes = {
  graphql: _propTypes.default.instanceOf(_GraphQL.GraphQL).isRequired,
  fetchOptionsOverride: _propTypes.default.func,
  operation: _propTypes.default.object.isRequired,
  loadOnMount: _propTypes.default.bool,
  loadOnReset: _propTypes.default.bool,
  resetOnLoad: _propTypes.default.bool,
  children: _propTypes.default.func.isRequired
}
