'use strict'

var _interopRequireDefault = require('@babel/runtime/helpers/interopRequireDefault')

exports.__esModule = true
exports.ssr = ssr

var _server = _interopRequireDefault(require('react-dom/server'))

var _GraphQL = require('./GraphQL')

function ssr(graphql, node, render) {
  var _arguments = arguments

  if (render === void 0) {
    render = _server.default.renderToStaticMarkup
  }

  return new Promise(function(resolve, reject) {
    if (!(graphql instanceof _GraphQL.GraphQL))
      throw new Error('ssr() argument 1 must be a GraphQL instance.')
    if (_arguments.length < 2)
      throw new Error('ssr() argument 2 must be a React node.')
    if (typeof render !== 'function')
      throw new Error('ssr() argument 3 must be a function.')
    graphql.ssr = true

    function recurse() {
      var fetching = []

      function onFetch(_ref) {
        var cache = _ref.cache
        fetching.push(cache)
      }

      graphql.on('fetch', onFetch)
      var string = render(node)
      graphql.off('fetch', onFetch)
      if (fetching.length) return Promise.all(fetching).then(recurse)
      else {
        delete graphql.ssr
        return Promise.resolve(string)
      }
    }

    return recurse()
      .then(resolve)
      .catch(reject)
  })
}
